import { RuntimeException } from '@poppinss/utils';
export class UrlBuilder {
    #qsParser;
    #params = {};
    #qs = {};
    #shouldPerformLookup = true;
    #baseUrl;
    #encryption;
    #routeFinder;
    constructor(encryption, routeFinder, qsParser) {
        this.#qsParser = qsParser;
        this.#encryption = encryption;
        this.#routeFinder = routeFinder;
    }
    #ensureHasWildCardValues(pattern, values) {
        if (!values || !Array.isArray(values) || !values.length) {
            throw new RuntimeException(`Cannot make URL for "${pattern}" route. Invalid value provided for wildcard param`);
        }
    }
    #ensureHasParamValue(pattern, param, value) {
        if (value === undefined || value === null) {
            throw new RuntimeException(`Cannot make URL for "${pattern}" route. Missing value for "${param}" param`);
        }
    }
    #processPattern(pattern) {
        const uriSegments = [];
        const paramsArray = Array.isArray(this.#params) ? this.#params : null;
        const paramsObject = !Array.isArray(this.#params) ? this.#params : {};
        let paramsIndex = 0;
        const tokens = pattern.split('/');
        for (const token of tokens) {
            if (token === '*') {
                const values = paramsArray ? paramsArray.slice(paramsIndex) : paramsObject['*'];
                this.#ensureHasWildCardValues(pattern, values);
                values.forEach((value) => uriSegments.push(value));
                break;
            }
            else if (!token.startsWith(':')) {
                uriSegments.push(token);
            }
            else {
                const paramName = token.replace(/^:/, '').replace(/\?$/, '');
                const value = paramsArray ? paramsArray[paramsIndex] : paramsObject[paramName];
                if (!token.endsWith('?')) {
                    this.#ensureHasParamValue(pattern, paramName, value);
                }
                paramsIndex++;
                if (value !== undefined && value !== null) {
                    uriSegments.push(value);
                }
            }
        }
        return uriSegments.join('/');
    }
    #suffixQueryString(url, qs) {
        if (qs) {
            const queryString = this.#qsParser.stringify(qs);
            url = queryString ? `${url}?${queryString}` : url;
        }
        return url;
    }
    #prefixBaseUrl(uri) {
        return this.#baseUrl ? `${this.#baseUrl}${uri}` : uri;
    }
    prefixUrl(url) {
        this.#baseUrl = url;
        return this;
    }
    disableRouteLookup() {
        this.#shouldPerformLookup = false;
        return this;
    }
    qs(queryString) {
        if (!queryString) {
            return this;
        }
        this.#qs = queryString;
        return this;
    }
    params(params) {
        if (!params) {
            return this;
        }
        this.#params = params;
        return this;
    }
    make(identifier) {
        let url;
        if (this.#shouldPerformLookup) {
            const route = this.#routeFinder.findOrFail(identifier);
            url = this.#processPattern(route.pattern);
        }
        else {
            url = this.#processPattern(identifier);
        }
        return this.#suffixQueryString(this.#prefixBaseUrl(url), this.#qs);
    }
    makeSigned(identifier, options) {
        let url;
        if (this.#shouldPerformLookup) {
            const route = this.#routeFinder.findOrFail(identifier);
            url = this.#processPattern(route.pattern);
        }
        else {
            url = this.#processPattern(identifier);
        }
        const signature = this.#encryption.verifier.sign(this.#suffixQueryString(url, this.#qs), options?.expiresIn, options?.purpose);
        const qs = Object.assign({}, this.#qs, { signature });
        return this.#suffixQueryString(this.#prefixBaseUrl(url), qs);
    }
}
