import string from '@poppinss/utils/string';
import { Macroable } from '@poppinss/macroable';
import { RuntimeException } from '@poppinss/utils';
import { Route } from './route.js';
export class RouteResource extends Macroable {
    #resource;
    #controller;
    #shallow = false;
    #globalMatchers;
    #app;
    #middlewareStore;
    #params = {};
    #routesBaseName;
    routes = [];
    constructor(app, middlewareStore, options) {
        super();
        this.#validateResourceName(options.resource);
        this.#app = app;
        this.#shallow = options.shallow;
        this.#middlewareStore = middlewareStore;
        this.#controller = options.controller;
        this.#globalMatchers = options.globalMatchers;
        this.#resource = this.#normalizeResourceName(options.resource);
        this.#routesBaseName = this.#getRoutesBaseName();
        this.#buildRoutes();
    }
    #normalizeResourceName(resource) {
        return resource.replace(/^\//, '').replace(/\/$/, '');
    }
    #validateResourceName(resource) {
        if (!resource || resource === '/') {
            throw new RuntimeException(`Invalid resource name "${resource}"`);
        }
    }
    #getRoutesBaseName() {
        return this.#resource
            .split('.')
            .map((token) => string.snakeCase(token))
            .join('.');
    }
    #createRoute(pattern, methods, action) {
        const route = new Route(this.#app, this.#middlewareStore, {
            pattern,
            methods,
            handler: `${this.#controller}.${action}`,
            globalMatchers: this.#globalMatchers,
        });
        route.as(`${this.#routesBaseName}.${action}`);
        this.routes.push(route);
    }
    #getResourceId(resource) {
        return `${string.snakeCase(string.singular(resource))}_id`;
    }
    #buildRoutes() {
        const resources = this.#resource.split('.');
        const mainResource = resources.pop();
        this.#params[mainResource] = ':id';
        const baseURI = `${resources
            .map((resource) => {
            const paramName = `:${this.#getResourceId(resource)}`;
            this.#params[resource] = paramName;
            return `${resource}/${paramName}`;
        })
            .join('/')}/${mainResource}`;
        this.#createRoute(baseURI, ['GET', 'HEAD'], 'index');
        this.#createRoute(`${baseURI}/create`, ['GET', 'HEAD'], 'create');
        this.#createRoute(baseURI, ['POST'], 'store');
        this.#createRoute(`${this.#shallow ? mainResource : baseURI}/:id`, ['GET', 'HEAD'], 'show');
        this.#createRoute(`${this.#shallow ? mainResource : baseURI}/:id/edit`, ['GET', 'HEAD'], 'edit');
        this.#createRoute(`${this.#shallow ? mainResource : baseURI}/:id`, ['PUT', 'PATCH'], 'update');
        this.#createRoute(`${this.#shallow ? mainResource : baseURI}/:id`, ['DELETE'], 'destroy');
    }
    #filter(names, inverse) {
        const actions = Array.isArray(names) ? names : [names];
        return this.routes.filter((route) => {
            const match = actions.find((name) => route.getName().endsWith(name));
            return inverse ? !match : match;
        });
    }
    only(names) {
        this.#filter(names, true).forEach((route) => route.markAsDeleted());
        return this;
    }
    except(names) {
        this.#filter(names, false).forEach((route) => route.markAsDeleted());
        return this;
    }
    apiOnly() {
        return this.except(['create', 'edit']);
    }
    where(key, matcher) {
        this.routes.forEach((route) => {
            route.where(key, matcher);
        });
        return this;
    }
    tap(actions, callback) {
        if (typeof actions === 'function') {
            this.routes.forEach((route) => actions(route));
            return this;
        }
        this.#filter(actions, false).forEach((route) => callback(route));
        return this;
    }
    params(resources) {
        Object.keys(resources).forEach((resource) => {
            const param = resources[resource];
            const existingParam = this.#params[resource];
            this.#params[resource] = `:${param}`;
            this.routes.forEach((route) => {
                route.setPattern(route.getPattern().replace(`${resource}/${existingParam}`, `${resource}/:${param}`));
            });
        });
        return this;
    }
    as(name) {
        name = string.snakeCase(name);
        this.routes.forEach((route) => {
            route.as(route.getName().replace(this.#routesBaseName, name), false);
        });
        this.#routesBaseName = name;
        return this;
    }
}
