import is from '@sindresorhus/is';
import Middleware from '@poppinss/middleware';
import { Macroable } from '@poppinss/macroable';
import { moduleExpression } from '@adonisjs/fold';
import { RuntimeException } from '@poppinss/utils';
import { execute } from './executor.js';
import { dropSlash } from '../helpers.js';
export class Route extends Macroable {
    #pattern;
    #methods;
    #name;
    #isDeleted = false;
    #handler;
    #globalMatchers;
    #app;
    #middlewareStore;
    #routeDomain = 'root';
    #matchers = {};
    #prefixes = [];
    #middleware = [];
    constructor(app, middlewareStore, options) {
        super();
        this.#app = app;
        this.#middlewareStore = middlewareStore;
        this.#pattern = options.pattern;
        this.#methods = options.methods;
        this.#handler = this.#resolveRouteHandle(options.handler);
        this.#globalMatchers = options.globalMatchers;
    }
    #resolveRouteHandle(handler) {
        if (typeof handler === 'string') {
            return {
                name: handler,
                ...moduleExpression(handler, this.#app.appRoot).toHandleMethod(),
            };
        }
        return handler;
    }
    #getMatchers() {
        return { ...this.#globalMatchers, ...this.#matchers };
    }
    #computePattern() {
        const pattern = dropSlash(this.#pattern);
        const prefix = this.#prefixes
            .slice()
            .reverse()
            .map((one) => dropSlash(one))
            .join('');
        return prefix ? `${prefix}${pattern === '/' ? '' : pattern}` : pattern;
    }
    where(param, matcher) {
        if (this.#matchers[param]) {
            return this;
        }
        if (typeof matcher === 'string') {
            this.#matchers[param] = { match: new RegExp(matcher) };
        }
        else if (is.regExp(matcher)) {
            this.#matchers[param] = { match: matcher };
        }
        else {
            this.#matchers[param] = matcher;
        }
        return this;
    }
    prefix(prefix) {
        this.#prefixes.push(prefix);
        return this;
    }
    domain(domain, overwrite = false) {
        if (this.#routeDomain === 'root' || overwrite) {
            this.#routeDomain = domain;
        }
        return this;
    }
    middleware(middleware, args) {
        if (typeof middleware === 'string') {
            this.#middleware.push([this.#middlewareStore.get(middleware, args)]);
            return this;
        }
        if (typeof middleware === 'function') {
            this.#middleware.push([middleware]);
        }
        return this;
    }
    as(name, prepend = false) {
        if (prepend) {
            if (!this.#name) {
                throw new RuntimeException(`Routes inside a group must have names before calling "router.group.as"`);
            }
            this.#name = `${name}.${this.#name}`;
            return this;
        }
        this.#name = name;
        return this;
    }
    isDeleted() {
        return this.#isDeleted;
    }
    markAsDeleted() {
        this.#isDeleted = true;
    }
    getName() {
        return this.#name;
    }
    getPattern() {
        return this.#pattern;
    }
    setPattern(pattern) {
        this.#pattern = pattern;
        return this;
    }
    getMiddleware() {
        return this.#middleware;
    }
    #getMiddlewareForStore() {
        const middleware = new Middleware();
        this.#middlewareStore.list().forEach((one) => middleware.add(one));
        this.#middleware.flat().forEach((one) => middleware.add(one));
        return middleware;
    }
    toJSON() {
        return {
            domain: this.#routeDomain,
            pattern: this.#computePattern(),
            matchers: this.#getMatchers(),
            meta: {},
            name: this.#name,
            handler: this.#handler,
            methods: this.#methods,
            middleware: this.#getMiddlewareForStore(),
            execute: execute,
        };
    }
}
